//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "afxsock.h"
#include "types.h"
#include "packets.h"
#include "AsyncSocketEx.h" // deadlake PROXYSUPPORT Socketfiles
#include "AsyncProxySocketLayer.h" // deadlake PROXYSUPPORT Socketfiles

#define ERR_WRONGHEADER		0x01
#define ERR_TOOBIG			0x02

#define	ES_DISCONNECTED		0xFF
#define	ES_NOTCONNECTED		0x00
#define	ES_CONNECTED		0x01
#define	ES_CONNECTING		0x02 //Pawcio: BC

#define PACKET_HEADER_SIZE	6

class CEMSocket : public CAsyncSocketEx // deadlake PROXYSUPPORT - changed to AsyncSocketEx
{
public:
	CEMSocket(void);
	~CEMSocket(void);

	//Pawcio: BC
	virtual bool	SendPacket(Packet* packet, bool delpacket = true, bool controlpacket = true, bool fileData = false);// controlpackets have a higher priority
	int		Send(char* lpBuf,int nBufLen,int nFlags = 0, bool fileData = false); //Pawcio: BC
	int		Receive(uint32 size, bool fileData = false);
    bool	IsBusy() {return sendbuffer;}
	bool	IsBlocked() const {return m_bWouldBlock;}
	bool	SendControlPacket(uint32 nMaxAmmount);
	uint32	GetControlPacketSizeToSend();
	uint32	GetBlockedDataSize() {return sendblen-sent;}
	bool	HasControlPacket() { return bCheckControlQueue;}
	void    SendBlockedData();
	void	ResetTimeOutTimer() {timeout_timer = ::GetTickCount();} //Pawcio: BC Moved from CClientReqSocket
	void	SetPrioritySend() {prioritySend = true;}
	bool	IsPrioritySend() {return prioritySend;}
	void	SetPriorityReceive(bool is) {priorityReceive = is;}
	bool	IsPriorityReceive() {return priorityReceive;}
	void	SetToRemove() {toRemove = true;}
	bool	IsSetToRemove() {return toRemove;}
	void	ClearLinkedPacket() {m_bLinkedPackets = false;}
	bool	IsOnDownControlQueue() {return onDownControlQueue;}
	bool	IsRemoveFromUpControl() {return toRemoveFromUpControl;}
	bool	IsRemoveFromDownControl() {return toRemoveFromDownControl;}
	bool	IsRemoveFromBlockedQueue() {return toRemoveFromBlockedQueue;}
	bool	IsOnUpControlQueue() {return onUpControlQueue;}
	bool	IsOnBlockedQueue() {return onBlockedQueue;}
	void	SetOnDownControlQueue(bool val) {onDownControlQueue=val;}
	void	SetOnUpControlQueue(bool val) {onUpControlQueue=val;}
	void	SetOnBlockedControlQueue(bool val) {onBlockedQueue=val;}
	void	SetRemoveFromUpControl(bool val) {toRemoveFromUpControl=val;}
	void	SetRemoveFromDownControl(bool val) {toRemoveFromDownControl=val;}
	void	SetRemoveFromBlockedQueue(bool val) {toRemoveFromBlockedQueue=val;}
	//Pawcio: BC
	bool	IsConnected() const {return byConnected == ES_CONNECTED;}
	uint8	GetConState() const {return byConnected;}
	//Pawcio: BC
	BOOL	AsyncSelect(long lEvent);

	// deadlake PROXYSUPPORT
	// By Maverick: Connection necessary initalizing calls are done by class itself and not anymore by the Owner
	BOOL	virtual Connect(LPCTSTR lpszHostAddress, UINT nHostPort);
	// Reset Layer Chain
	virtual void RemoveAllLayers();

protected:
	//Pawcio: BC
	void SplitHeadControlPacket();
	uint32	timeout_timer; //Pawcio: BC Moved from CClientReqSocket
	virtual void    OnConnect(int nErrorCode);
	virtual void	OnConnectError(int nErrorCode) {};
	bool	toRemove;
	bool	onDownControlQueue;
	bool	onUpControlQueue;
	bool	onBlockedQueue;
	bool	toRemoveFromUpControl;
	bool	toRemoveFromDownControl;
	bool	toRemoveFromBlockedQueue;
	bool	prioritySend;
	bool	priorityReceive;
	//Pawcio: BC
	virtual int	OnLayerCallback(const CAsyncSocketExLayer *pLayer, int nType, int nParam1, int nParam2);	// deadlake PROXYSUPPORT
	virtual void	PacketReceived(Packet* packet) = 0;
	virtual void	OnError(int nErrorCode) = 0;
	virtual void	OnClose(int nErrorCode);
	virtual void	OnSend(int nErrorCode);	
	virtual void	OnReceive(int nErrorCode);
	uint8	byConnected;

	// deadlake PROXYSUPPORT
	bool	m_ProxyConnectFailed;
	CAsyncProxySocketLayer* m_pProxyLayer;

private:
	void	ClearQueues();	
	//Pawcio: BC
	virtual int Receive(void* lpBuf, int nBufLen, int nFlags = 0, bool fileData = false);

	//Pawcio: BC

	// Download partial header
	char*	pendingHeader[PACKET_HEADER_SIZE];      
	uint32	pendingHeaderSize;

	// Download partial packet
	Packet* pendingPacket;
	uint32  pendingPacketSize;

	// Upload control
	char*	sendbuffer;
	uint32	sendblen;
	uint32	sent;
	bool	m_bLinkedPackets;
	//Pawcio: BC
	bool	m_bWouldBlock;
	bool	bCheckControlQueue;
	bool	fileData;
	//Pawcio: BC

	CTypedPtrList<CPtrList, Packet*> controlpacket_queue;
	//CTypedPtrList<CPtrList, Packet*> standartpacket_queue; //Pawcio: BC
};
